﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Apache.NMS;
using Apache.NMS.Util;
using Apache.NMS.Stomp;
using StompCommands = Apache.NMS.Stomp.Commands;


namespace HornetQIssue
{
    class Interaction : IDisposable
    {

        IConnectionFactory factory { get; set; }
        IConnection connection { get; set; }
        ISession session { get; set; }
        IMessageConsumer consumerA { get; set; }
        IMessageConsumer consumerB { get; set; }
        System.Threading.EventWaitHandle signal = new System.Threading.EventWaitHandle(false, System.Threading.EventResetMode.ManualReset);
        string[] waitForMessages { get; set; }
        string[] ackForMessages { get; set; }
        string logInfo { get; set; }

        public void Dispose()
        {
            Disconnect();
        }

        protected void Disconnect()
        {
            if (consumerA != null)
            {
                consumerA.Close();
//                consumerA.Dispose();
                consumerA = null;
            }

            if (consumerB != null)
            {
                consumerB.Close();
//                consumerB.Dispose();
                consumerB = null;
            }

            if (session != null)
            {
                session.Close();
//                session.Dispose();
                session = null;
            }

            if (connection != null)
            {
                connection.Stop();
                connection.Close();
//                connection.Dispose();
                connection = null;
            }

            factory = null;
        }

        protected void Connect(string serverName)
        {
            Disconnect();
            //Create the Connection factory 
            factory = new ConnectionFactory(serverName);
            //Create the connection 
            connection = factory.CreateConnection();
            connection.AcknowledgementMode = AcknowledgementMode.ClientAcknowledge;
            connection.Start();

            //Create the Session 
            session = connection.CreateSession();
        }

        virtual public void Listener(IMessage message)
        {
            ITextMessage msg = message as ITextMessage;

            Console.Write("{0}: {1}", this.logInfo, msg.Text);

            this.waitForMessages = (from waitForMessage in this.waitForMessages where waitForMessage != msg.Text select waitForMessage).ToArray();
            if (this.ackForMessages.Contains(msg.Text))
            {
                message.Acknowledge();
                Console.WriteLine(" (ACKnowledged)");
            }
            else
            {
                Console.WriteLine(string.Empty);
            }

            if (this.waitForMessages.Length == 0)
            {
                signal.Set();
            }
        }

        public bool Signaled(int wait)
        {
            return signal.WaitOne(wait);
        }


        public void Listen(string serverName, string queueName, string[] waitForMessages, string[] ackForMessages, string logInfo)
        {
            if (consumerA == null)
            {
                this.signal.Reset();
                this.logInfo = logInfo;
                this.Connect(serverName);
                this.ackForMessages = ackForMessages;
                this.waitForMessages = waitForMessages;
                this.consumerA = session.CreateConsumer(new Apache.NMS.Stomp.Commands.Queue(queueName), null);
                this.consumerA.Listener += new MessageListener(this.Listener);
            }
            else
            {
                this.consumerB = session.CreateConsumer(new Apache.NMS.Stomp.Commands.Queue(queueName), null);
                this.consumerB.Listener += new MessageListener(this.Listener);
            }
        }

        public string Send(string serverName, string queueName, string message)
        {
            Connect(serverName);
            IMessageProducer producer = null;
            //Create the Producer for the queue 
            producer = session.CreateProducer(new Apache.NMS.Stomp.Commands.Queue(queueName));
            //Send Messages 
            ITextMessage msg = producer.CreateTextMessage(message);
            producer.Send(msg, MsgDeliveryMode.Persistent, MsgPriority.Normal, TimeSpan.MinValue);
            return msg.NMSMessageId;
        }
    }
}
